@@ -34,11 +34,11 @@ Data::ICal::RDF - Turn iCal files into an RDF graph
 
 =head1 VERSION
 
-Version 0.01
+Version 0.03
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 # built-in ref types for our robust type checker
 my %CORE = map { $_ => 1 } qw(SCALAR ARRAY HASH CODE REF GLOB LVALUE
@@ -218,10 +218,10 @@ my %VALS = (
         }
 
         # turn the val into an IO object
-        $val = IO::Scalar->new(\$val);
+        my $io = IO::Scalar->new(\$val);
 
         # now try to resolve the attachment
-        my $o = eval { $self->resolve_binary->($self, $val, $type, $name) };
+        my $o = eval { $self->resolve_binary->($self, $io, $type, $name) };
         $self->throw("resolve_binary callback failed: $@") if $@;
         $self->throw('resolve_binary callback returned an invalid value')
               unless _is_really($o, 'RDF::Trine::Node');
@@ -265,8 +265,11 @@ my %VALS = (
         my $dt  = DateTime::Format::ICal->parse_datetime($prop->value);
 
         my $tzid = $prop->parameters->{TZID};
+        #warn "TZID: $tzid" if $tzid;
+        #require Data::Dumper;
+        #warn Data::Dumper::Dumper($self->tz);
         if ($tzid and my $tz = $self->tz->{$tzid}) {
-            warn 'hooray that whole effort worked!';
+            #warn 'hooray that whole effort worked!';
             $dt->set_time_zone($tz);
         }
 
@@ -337,7 +340,7 @@ my %VALS = (
 
         my $uri = URI->new($prop->value)->canonical;
         my $p = $self->_predicate_for($prop);
-        $self->model->add_statement($s, $p, iri($uri->as_string));
+        $self->model->add_statement(statement($s, $p, iri($uri->as_string)));
 
         $uri;
     },
@@ -417,9 +420,17 @@ sub _process_property {
     # Now you can do whatever you like with the model.
     my $result = $context->model;
 
+=head1 DESCRIPTION
+
+This module is a processor context for turning L<Data::ICal> objects
+into RDF data. By default it uses version 4 (i.e., random) UUIDs as
+subject nodes.
+
 =head1 METHODS
 
-=head2 new
+=head2 new %PARAMS
+
+Initialize the processor context.
 
 =over 4
 
@@ -447,6 +458,14 @@ L</process_events>, which is used by L</process>. If the function is
 not reliable for any reason, such as a failure to access hardware or
 network resources, those methods may C<croak>.
 
+By default the processor will automatically convert iCal UIDs which
+are V4 UUIDs into C<urn:uuid:> URIs and use them as the subjects of
+the resulting RDF statements. Furthermore, this is checked I<before>
+running this function to mitigate any database overhead (see
+L</no_uuids>). A V4 UUID URN is also generated as the iCal data's
+subject if this function returns C<undef>. If you do I<not> want to
+use UUIDs, then this function must I<always> return a valid value.
+
 Here is an example of a method in a fictitious class which generates a
 closure suitable to pass into the L<Data::ICal::RDF> constructor:
 
@@ -590,10 +609,21 @@ has tz => (
     default => sub { { } },
 );
 
+=item no_uuids
+
+This is a flag to alter the short-circuiting behaviour of
+L</subject_for>. When set, it will I<not> attempt to return the result
+of L</uid_is_uuid> before running L</resolve_uid>.
+
 =back
 
 =cut
 
+has no_uuids => (
+    is => 'rw',
+    default => sub { 0 },
+);
+
 has _subjects => (
     is => 'ro',
     default => sub { { } },
@@ -626,8 +656,12 @@ sub process {
         # snag all the time zones
         if ($t eq 'VTIMEZONE') {
             my $dtz = DateTime::TimeZone::ICal->from_ical_entry($entry);
-            $self->tz->{$dtz->name} = $dtz;
-            # XXX create a timezone object?
+            # woops, looks like DateTime::TimeZone aliasing messes
+            # with the name and causes time zones to be unfindable
+            my $id = $entry->property('TZID')->[0]->value;
+            $self->tz->{$id} = $dtz;
+
+            # XXX should we create a timezone object in rdf?
         }
         elsif ($t eq 'VEVENT') {
             push @events, $entry;
@@ -663,6 +697,10 @@ sub process_events {
         my $s = eval { $self->subject_for($uid->value) };
         $self->throw($@) if $@;
 
+        # don't forget to add the uid
+        $self->model->add_statement(statement(
+            $s, $NS->ical->uid, literal($uid->value, undef, $NS->xsd->string)));
+
         # don't forget to add the type
         $self->model->add_statement
             (statement($s, $NS->rdf->type, $NS->ical->Vevent));
@@ -709,11 +747,7 @@ therefore may croak if it receives a bad value.
 sub subject_for {
     my ($self, $uid) = @_;
 
-    # check to see if this is a V4 UUID
-    if (my @parts = ($uid =~ $UUID4)) {
-        # if it is, convert it into a resource node and return it
-        my $s = iri('urn:uuid:' . lc join '-', @parts);
-        #warn "$s is already a V4 UUID";
+    if (!$self->no_uuids and my $s = $self->uid_is_uuid($uid)) {
         return $s;
     }
 
@@ -725,15 +759,16 @@ sub subject_for {
 
     # call out to the callback
     if (my $s = eval { $self->resolve_uid->($self, $uid) }) {
-        $self->throw("resolve_uid callback failed: $@") if $@;
         $self->throw('resolve_uid callback returned an invalid value')
-              unless _is_really($s, 'RDF::Trine::Node');
+            unless _is_really($s, 'RDF::Trine::Node');
         $self->throw("Node $s returned from resolve_uid callback" .
-                        ' is not suitable as a subject')
-              unless ($s->is_resource or $s->is_blank);
-
+                             ' is not suitable as a subject')
+            unless ($s->is_resource or $s->is_blank);
         return $self->_subjects->{$uid} = $s;
     }
+    # explode if the eval failed
+    $self->throw("resolve_uid callback failed: $@") if $@;
+
 
     # if we can't find a cached entry or a mapping in the database,
     # then we create one from scratch (and cache it).
@@ -742,14 +777,42 @@ sub subject_for {
     return $self->_subjects->{$uid} = $s;
 }
 
+=head2 uuid_is_uid $UID
+
+Returns a suitable C<urn:uuid:> node if the iCal UID is also a valid
+(version 4) UUID. Used by L</subject_for> and available in the
+L<resolve_uid> and L<resolve_binary> functions.
+
+=cut
+
+sub uid_is_uuid {
+    my ($self, $uid) = @_;
+
+    # check to see if this is a V4 UUID
+    if (my @parts = ($uid =~ $UUID4)) {
+        # if it is, convert it into a resource node and return it
+        my $s = iri('urn:uuid:' . lc join '-', @parts);
+        #warn "$s is already a V4 UUID";
+        return $s;
+    }
+}
+
 =head2 model
 
 Retrieve the L<RDF::Trine::Model> object embedded in the processor.
 
 =head1 CAVEATS
 
-A number of iCal datatype handlers are not implemented in this early
-version. These are:
+This module is I<prototype-grade>, and may give you unexpected
+results. It does not have a test suite to speak of, at least not until
+I can come up with an adequate one. An exhaustive test suite to handle
+the vagaries of the iCal format would likely take an order of
+magnitude more effort than the module code itself. Nevertheless, I
+know it works because I'm using it, so my "test suite" is production.
+I repeat, this is I<not> mature software. Patches welcome.
+
+Furthermore, a number of iCal datatype handlers are not implemented in
+this early version. These are:
 
 =over 4
 
@@ -779,6 +842,12 @@ C<UTC-OFFSET>
 
 =back
 
+In particular, a lack of a handler for the C<DURATION> type means
+events that follow the C<DTSTART>/C<DURATION> form will be incomplete.
+In practice this should not be a problem, as iCal, Outlook, etc. use
+C<DTEND>. This is also in part a design issue, as to whether the
+C<DURATION> I<property> should be normalized to C<DTEND>.
+
 As well, the C<GEO>, C<RESOURCES>, and C<CLASS> properties are yet to
 be implemented. Patches are welcome, as are work orders.
 
